@@ -29,12 +29,14 @@
 #include <gtk/gtk.h>
 #include <glib-object.h>
 #include "debug.h"
+
+#undef VTE_DISABLE_DEPRECATED
 #include "vte.h"
 
 #include <glib/gi18n.h>
 
-#define DINGUS1 "(((news|telnet|nntp|file|http|ftp|https)://)|(www|ftp)[-A-Za-z0-9]*\\.)[-A-Za-z0-9\\.]+(:[0-9]*)?"
-#define DINGUS2 "(((news|telnet|nntp|file|http|ftp|https)://)|(www|ftp)[-A-Za-z0-9]*\\.)[-A-Za-z0-9\\.]+(:[0-9]*)?/[-A-Za-z0-9_\\$\\.\\+\\!\\*\\(\\),;:@&=\\?/~\\#\\%]*[^]'\\.}>\\) ,\\\"]"
+#define DINGUS1 "(((gopher|news|telnet|nntp|file|http|ftp|https)://)|(www|ftp)[-A-Za-z0-9]*\\.)[-A-Za-z0-9\\.]+(:[0-9]*)?"
+#define DINGUS2 "(((gopher|news|telnet|nntp|file|http|ftp|https)://)|(www|ftp)[-A-Za-z0-9]*\\.)[-A-Za-z0-9\\.]+(:[0-9]*)?/[-A-Za-z0-9_\\$\\.\\+\\!\\*\\(\\),;:@&=\\?/~\\#\\%]*[^]'\\.}>\\) ,\\\"]"
 
 static void
 window_title_changed(GtkWidget *widget, gpointer win)
@@ -128,29 +130,49 @@ char_size_realized(GtkWidget *widget, gpointer data)
 				      GDK_HINT_MIN_SIZE);
 }
 
-static void
-deleted_and_quit(GtkWidget *widget, GdkEvent *event, gpointer data)
-{
-	gtk_widget_destroy(GTK_WIDGET(data));
-	gtk_main_quit();
-}
 
 static void
-destroy_and_quit(GtkWidget *widget, gpointer data)
+destroy_and_quit(VteTerminal *terminal, GtkWidget *window)
 {
-	gtk_widget_destroy(GTK_WIDGET(data));
-	gtk_main_quit();
+	const char *output_file = g_object_get_data (G_OBJECT (terminal), "output_file");
+
+	if (output_file) {
+		GFile *file;
+		GOutputStream *stream;
+		GError *error = NULL;
+
+		file = g_file_new_for_commandline_arg (output_file);
+		stream = G_OUTPUT_STREAM (g_file_replace (file, NULL, FALSE, G_FILE_CREATE_NONE, NULL, &error));
+
+		if (stream) {
+			vte_terminal_write_contents (terminal, stream,
+						     VTE_TERMINAL_WRITE_DEFAULT,
+						     NULL, &error);
+			g_object_unref (stream);
+		}
+
+		if (error) {
+			g_printerr ("%s\n", error->message);
+			g_error_free (error);
+		}
+
+		g_object_unref (file);
+	}
+
+	gtk_widget_destroy (window);
+	gtk_main_quit ();
 }
 static void
-destroy_and_quit_eof(GtkWidget *widget, gpointer data)
+delete_event(GtkWidget *window, GdkEvent *event, gpointer terminal)
 {
-	_vte_debug_print(VTE_DEBUG_MISC, "Detected EOF.\n");
+	destroy_and_quit(VTE_TERMINAL (terminal), window);
 }
 static void
-destroy_and_quit_exited(GtkWidget *widget, gpointer data)
+child_exited(GtkWidget *terminal, gpointer window)
 {
-	_vte_debug_print(VTE_DEBUG_MISC, "Detected child exit.\n");
-	destroy_and_quit(widget, data);
+	_vte_debug_print(VTE_DEBUG_MISC, "Child exited with status %x\n",
+			 vte_terminal_get_child_exit_status (VTE_TERMINAL (terminal)));
+	destroy_and_quit(VTE_TERMINAL (terminal), GTK_WIDGET (window));
 }
 
 static void
@@ -172,9 +194,9 @@ button_pressed(GtkWidget *widget, GdkEventButton *event, gpointer data)
 		terminal = VTE_TERMINAL(widget);
 		vte_terminal_get_padding(terminal, &xpad, &ypad);
 		match = vte_terminal_match_check(terminal,
-						 (event->x - ypad) /
+						 (event->x - xpad / 2) /
 						 terminal->char_width,
-						 (event->y - ypad) /
+						 (event->y - ypad / 2) /
 						 terminal->char_height,
 						 &tag);
 		if (match != NULL) {
@@ -432,16 +454,22 @@ terminal_notify_cb(GObject *object,
   g_value_unset(&value);
 }
 
+static void
+child_exit_cb(VteTerminal *terminal,
+                 gpointer user_data)
+{
+}
+
 int
 main(int argc, char **argv)
 {
 	GdkScreen *screen;
 	GdkColormap *colormap;
-	GtkWidget *window, *widget,*hbox, *scrollbar, *scrolled_window;
+	GtkWidget *window, *widget,*hbox = NULL, *scrollbar, *scrolled_window = NULL;
 	VteTerminal *terminal;
 	char *env_add[] = {
 #ifdef VTE_DEBUG
-		"FOO=BAR", "BOO=BIZ",
+		(char *) "FOO=BAR", (char *) "BOO=BIZ",
 #endif
 		NULL};
 	const char *background = NULL;
@@ -460,6 +488,8 @@ main(int argc, char **argv)
 	const char *termcap = NULL;
 	const char *command = NULL;
 	const char *working_directory = NULL;
+	const char *output_file = NULL;
+        char *cursor_shape = NULL;
 	GdkColor fore, back, tint, highlight, cursor;
 	const GOptionEntry options[]={
 		{
@@ -553,6 +583,11 @@ main(int argc, char **argv)
 			G_OPTION_ARG_NONE, &cursor_set,
 			"Enable a colored cursor", NULL
 		},
+		{
+			"cursor-shape", 0, 0,
+			G_OPTION_ARG_STRING, &cursor_shape,
+			"Set cursor shape (block, underline, ibeam)", NULL
+		},
 		{
 			"scroll-background", 's', 0,
 			G_OPTION_ARG_NONE, &scroll,
@@ -598,6 +633,11 @@ main(int argc, char **argv)
 			"Print VteTerminal object notifications",
 			NULL
 		},
+		{
+			"output-file", 0, 0,
+			G_OPTION_ARG_STRING, &output_file,
+			"Save terminal contents to file at exit", NULL
+		},
 		{ NULL }
 	};
 	GOptionContext *context;
@@ -643,8 +683,6 @@ main(int argc, char **argv)
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_container_set_resize_mode(GTK_CONTAINER(window),
 				      GTK_RESIZE_IMMEDIATE);
-	g_signal_connect(window, "delete-event",
-			 G_CALLBACK(deleted_and_quit), window);
 
 	/* Set ARGB colormap */
 	screen = gtk_widget_get_screen (window);
@@ -670,6 +708,7 @@ main(int argc, char **argv)
 	if (!dbuffer) {
 		gtk_widget_set_double_buffered(widget, dbuffer);
 	}
+        g_signal_connect(terminal, "child-exited", G_CALLBACK(child_exit_cb), NULL);
         if (show_object_notifications)
                 g_signal_connect(terminal, "notify", G_CALLBACK(terminal_notify_cb), NULL);
         if (use_scrolled_window) {
@@ -697,12 +736,6 @@ main(int argc, char **argv)
 				 G_CALLBACK(icon_title_changed), window);
 	}
 
-	/* Connect to the "eof" signal to quit when the session ends. */
-	g_signal_connect(widget, "eof",
-			 G_CALLBACK(destroy_and_quit_eof), window);
-	g_signal_connect(widget, "child-exited",
-			 G_CALLBACK(destroy_and_quit_exited), window);
-
 	/* Connect to the "status-line-changed" signal. */
 	g_signal_connect(widget, "status-line-changed",
 			 G_CALLBACK(status_line_changed), widget);
@@ -770,6 +803,13 @@ main(int argc, char **argv)
 	if (cursor_set) {
 		vte_terminal_set_color_cursor(terminal, &cursor);
 	}
+        if (g_strcmp0(cursor_shape, "underline") == 0) {
+                vte_terminal_set_cursor_shape(terminal, VTE_CURSOR_SHAPE_UNDERLINE);
+        } else if (g_strcmp0(cursor_shape, "ibeam") == 0) {
+                vte_terminal_set_cursor_shape(terminal, VTE_CURSOR_SHAPE_IBEAM);
+        } else {
+                vte_terminal_set_cursor_shape(terminal, VTE_CURSOR_SHAPE_BLOCK);
+        }
 	if (termcap != NULL) {
 		vte_terminal_set_emulation(terminal, termcap);
 	}
@@ -779,13 +819,17 @@ main(int argc, char **argv)
 					       antialias ? VTE_ANTI_ALIAS_USE_DEFAULT : VTE_ANTI_ALIAS_FORCE_DISABLE);
 
 	/* Match "abcdefg". */
-	vte_terminal_match_add(terminal, "abcdefg");
 	if (dingus) {
 		int id;
-		id = vte_terminal_match_add(terminal, DINGUS1);
+		GRegex *regex;
+		regex = g_regex_new (DINGUS1, 0, 0, NULL);
+		id = vte_terminal_match_add_gregex(terminal, regex, 0);
+		g_regex_unref (regex);
 		vte_terminal_match_set_cursor_type(terminal,
 						   id, GDK_GUMBY);
-		id = vte_terminal_match_add(terminal, DINGUS2);
+		regex = g_regex_new (DINGUS2, 0, 0, NULL);
+		id = vte_terminal_match_add_gregex(terminal, regex, 0);
+		g_regex_unref (regex);
 		vte_terminal_match_set_cursor_type(terminal,
 						   id, GDK_HAND1);
 	}
@@ -886,7 +930,12 @@ main(int argc, char **argv)
 		}
 	}
 
+	g_object_set_data (G_OBJECT (widget), "output_file", (gpointer) output_file);
+
 	/* Go for it! */
+	g_signal_connect(widget, "child-exited", G_CALLBACK(child_exited), window);
+	g_signal_connect(window, "delete-event", G_CALLBACK(delete_event), widget);
+
 	add_weak_pointer(G_OBJECT(widget), &widget);
 	add_weak_pointer(G_OBJECT(window), &window);
 
@@ -899,7 +948,6 @@ main(int argc, char **argv)
 
 	gtk_widget_show_all(window);
 
-
 	gtk_main();
 
 	g_assert(widget == NULL);
